////////////////////////////////////////////////////////////////////////////
//	Module 		: enemy_manager.h
//	Created 	: 30.12.2003
//  Modified 	: 30.12.2003
//	Author		: Dmitriy Iassenev
//	Description : Enemy manager
////////////////////////////////////////////////////////////////////////////

#pragma once

#include "object_manager.h"
#include "entity_alive.h"
#include "custommonster.h"
#include "script_callback_ex.h"

class CEnemyManager : public CObjectManager<const CEntityAlive> {
public:
	typedef CObjectManager<const CEntityAlive>	inherited;
	typedef OBJECTS								ENEMIES;
	typedef CScriptCallbackEx<bool>				USEFULE_CALLBACK;

private:
	CCustomMonster				*m_object;
	float						m_ignore_monster_threshold;
	float						m_max_ignore_distance;
	mutable bool				m_ready_to_save;
	mutable bool				m_visible_now;
	u32							m_last_enemy_time;
	const CEntityAlive			*m_last_enemy;
	USEFULE_CALLBACK			m_useful_callback;

private:
	u32							m_last_enemy_change;

protected:
			void				on_enemy_change		(const CEntityAlive *previous_enemy);
			bool				expedient			(const CEntityAlive *object) const;

public:
	IC							CEnemyManager		(CCustomMonster *object);
	virtual void				reload				(LPCSTR section);
	virtual bool				useful				(const CEntityAlive *object) const;
	virtual bool				is_useful			(const CEntityAlive *object) const;
	virtual	float				evaluate			(const CEntityAlive *object) const;
	virtual	float				do_evaluate			(const CEntityAlive *object) const;
	virtual void				update				();
	virtual void				set_ready_to_save	();
	IC		u32					last_enemy_time		() const;
	IC		const CEntityAlive	*last_enemy			() const;
	IC		USEFULE_CALLBACK	&useful_callback	();
			void				remove_links		(CObject *object);

public:
			void				ignore_monster_threshold			(const float &ignore_monster_threshold);
			void				restore_ignore_monster_threshold	();
			float				ignore_monster_threshold			() const;
			void				max_ignore_monster_distance			(const float &max_ignore_monster_distance);
			void				restore_max_ignore_monster_distance	();
			float				max_ignore_monster_distance			() const;

};

#include "enemy_manager_inline.h"